/// @addtogroup dynset
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file c1rect.hpp
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _C1RECT_HPP
#define _C1RECT_HPP

#include <sstream>
#include "dynset/c1rectintf.h"

namespace capd{
namespace dynset{

template<typename MatrixType>
C1rect<MatrixType>::C1rect(const NormType& theNorm, int dim)
   :  m_x(dim),
      m_r(dim),
      m_B(MatrixType::Identity(dim)),
      m_D(MatrixType::Identity(dim)),
      m_Bjac(MatrixType::Identity(dim)),
      m_R(dim,dim),
      m_Norm(theNorm.clone())
{
   m_x.clear();
   m_r.clear();
   m_R.clear();
}

template<typename MatrixType>
C1rect<MatrixType>::C1rect(const C1rect<MatrixType>& s)
   :  m_x(s.m_x),
      m_r(s.m_r),
      m_B(s.m_B),
      m_D(s.m_D),
      m_Bjac(s.m_Bjac),
      m_R(s.m_R),
      m_Norm(s.m_Norm->clone())
{}

template<typename MatrixType>
C1rect<MatrixType>::C1rect(const VectorType& theX, const NormType& theNorm)
   :  m_x(theX),
      m_r(theX.dimension()),
      m_B(MatrixType::Identity(theX.dimension())),
      m_D(MatrixType::Identity(theX.dimension())),
      m_Bjac(MatrixType::Identity(theX.dimension())),
      m_R(theX.dimension(),theX.dimension()),
      m_Norm(theNorm.clone())
{
   split(m_x,m_r);
   m_R.clear();
}

template<typename MatrixType>
C1rect<MatrixType>::C1rect(const VectorType& theX,const VectorType& the_r,const NormType& theNorm)
   :  m_x(theX),
      m_r(the_r),
      m_B(MatrixType::Identity(theX.dimension())),
      m_D(MatrixType::Identity(theX.dimension())),
      m_Bjac(MatrixType::Identity(theX.dimension())),
      m_R(theX.dimension(),theX.dimension()),
      m_Norm(theNorm.clone())
{
   m_R.clear();
}


template<typename MatrixType>
C1rect<MatrixType>::C1rect(
      const VectorType& theX,
      const MatrixType& theB,
      const VectorType& the_r,
      const NormType& theNorm
   ): m_x(theX),
      m_r(the_r),
      m_B(theB),
      m_D(MatrixType::Identity(theX.dimension())),
      m_Bjac(MatrixType::Identity(theX.dimension())),
      m_R(theX.dimension(),theX.dimension()),
      m_Norm(theNorm.clone())
{
   m_R.clear();
}

template<typename MatrixType>
void C1rect<MatrixType>::move(capd::dynsys::C1DynSys<MatrixType>& c1dynsys, C1rect& result) const
{
   int dim = m_x.dimension();

   VectorType z(dim), Rem(dim);
   MatrixType Z(dim,dim), delta_A(dim,dim), jacRem(dim,dim);

   VectorType xx = m_x + m_B*m_r;
   VectorType y = c1dynsys.Phi(m_x);
   split(y,z);

   int found;
   VectorType Enc = c1dynsys.enclosure(xx,&found);
   MatrixType jacEnc = c1dynsys.jacEnclosure(Enc,*m_Norm);
   c1dynsys.JacRemainder(Enc,jacEnc,Rem,jacRem);
   z += Rem;

   MatrixType Jac = c1dynsys.JacPhi(xx);
   MatrixType X = Jac*m_B;
   MatrixType Q = midMatrix(X);
   orthonormalize(Q);
   MatrixType Qtr = Transpose(Q);
   result.m_x = y;
   result.m_r = (Qtr*X)*m_r+Qtr*z;
   result.m_B = Q;


   MatrixType A = Jac+jacRem;

   MatrixType D1 = m_D + m_Bjac * m_R;
   splitMatrix(A,delta_A);

   MatrixType Y = A*m_D + delta_A*D1;
   splitMatrix(Y,Z);
   result.m_D = Y;
   X = A*m_Bjac;
   Q = midMatrix(X);
   orthonormalize(Q);
   Qtr = Transpose(Q);
   result.m_R = (Qtr*X)*m_R + Qtr*Z;
   result.m_Bjac = Q;
}

// ----------------------------------------------

template<typename MatrixType>
typename C1rect<MatrixType>::ScalarType C1rect<MatrixType>::size(void) const
{
   return ::size(m_x + m_B*m_r);
}

template<typename MatrixType>
std::string C1rect<MatrixType>::show(void) const
{
   std::ostringstream descriptor;
   descriptor << "C1rect:\n x=";
   descriptor << m_x << "\n B=";
   descriptor << m_B << "\n r=";
   descriptor << m_r << "\n D= ";
   descriptor << m_D << "\n Bjac= ";
   descriptor << m_Bjac << "\n R= ";
   descriptor << m_R << '\0';

   return descriptor.str();
}

template<typename MatrixT>
C1rect<MatrixT>::operator typename C1rect<MatrixT>::MatrixType() const
{
   return m_D + m_Bjac*m_R;
}

template<typename MatrixType>
C1rect<MatrixType> &C1rect<MatrixType>::operator=(const VectorType& theX)
{
   int dim = theX.dimension();
   m_x = theX;
   m_r = VectorType(dim);
   split(m_x,m_r);

   m_Bjac = m_D = m_B = MatrixType::Identity(dim);
   m_R = MatrixType(dim,dim);
   m_R.clear();

   return *this;
}

template<typename MatrixType>
C1rect<MatrixType>& C1rect<MatrixType>::operator=(const C1rect& S)
{
   m_x = S.m_x;
   m_B = S.m_B;
   m_r = S.m_r;

   m_D = S.m_D;
   m_Bjac = S.m_Bjac;
   m_R = S.m_R;
   delete m_Norm;
   m_Norm = S.m_Norm->clone();
   return *this;
}

}} // namespace capd::dynset

#endif // _C1RECT_HPP

/// @}
