/// @addtogroup dynset
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file c1rect2.hpp
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _C1RECT_HPP
#define _C1RECT_HPP

#include <sstream>
#include "capd/minmax.h"
#include "dynset/c1rect2intf.h"

namespace capd{
namespace dynset{

template<typename MatrixType>
C1rect2<MatrixType>::C1rect2(const NormType &the_norm, int dim)
   :  m_x(dim), m_r(dim), m_r0(dim),
      m_B(MatrixType::Identity(dim)),
      m_C(MatrixType::Identity(dim)),
      m_D(MatrixType::Identity(dim)),
      m_Bjac(MatrixType::Identity(dim)),
      m_R(dim,dim),
      m_Cjac(MatrixType::Identity(dim)),
      m_R0(dim,dim),
      m_Norm(the_norm.clone()),
      m_sizeFactor(20.)
{
   m_x.clear();
   m_r.clear();
   m_r0.clear();
   m_R.clear();
   m_R0.clear();
}


template<typename MatrixType>
C1rect2<MatrixType>::C1rect2(const C1rect2 &s)
   :  m_x(s.m_x),
      m_r(s.m_r),
      m_r0(s.m_r0),
      m_B(s.m_B),
      m_C(s.m_C),
      m_D(s.m_D),
      m_Bjac(s.m_Bjac),
      m_R(s.m_R),
      m_Cjac(s.m_Cjac),
      m_R0(s.m_R0),
      m_Norm(s.m_Norm->clone()),
      m_sizeFactor(s.m_sizeFactor)
{}



template<typename MatrixType>
C1rect2<MatrixType>::C1rect2(const VectorType &theX, const NormType &theNorm, double sFactor)
   :   m_x(theX),
       m_r(theX.dimension()),
       m_r0(theX.dimension()),
       m_B(MatrixType::Identity(theX.dimension())),
       m_C(MatrixType::Identity(theX.dimension())),
       m_D(MatrixType::Identity(theX.dimension())),
       m_Bjac(MatrixType::Identity(theX.dimension())),
       m_R(theX.dimension(),theX.dimension()),
       m_Cjac(MatrixType::Identity(theX.dimension())),
       m_R0(theX.dimension(),theX.dimension()),
       m_Norm(theNorm.clone()),
       m_sizeFactor(sFactor)
{
   split(m_x,m_r0);
   m_r.clear();
   m_R.clear();
   m_R0.clear();
}

template<typename MatrixType>
C1rect2<MatrixType>::C1rect2(
      const VectorType& theX,
      const VectorType& the_r0,
      const NormType& theNorm,
      double sFactor
   ) : m_x(theX),
       m_r(theX.dimension()),
       m_r0(the_r0),
       m_B(MatrixType::Identity(theX.dimension())),
       m_C(MatrixType::Identity(theX.dimension())),
       m_D(MatrixType::Identity(theX.dimension())),
       m_Bjac(MatrixType::Identity(theX.dimension())),
       m_R(theX.dimension(),theX.dimension()),
       m_Cjac(MatrixType::Identity(theX.dimension())),
       m_R0(theX.dimension(),theX.dimension()),
       m_Norm(theNorm.clone()),
       m_sizeFactor(sFactor)
{
   m_r.clear();
   m_R.clear();
   m_R0.clear();
}


template<typename MatrixType>
C1rect2<MatrixType>::C1rect2(
      const VectorType &theX,
      const MatrixType &theC,
      const VectorType &the_r0,
      const NormType &theNorm,
      double sFactor
   ) : m_x(theX),
       m_r(theX.dimension()),
       m_r0(the_r0),
       m_B(MatrixType::Identity(theX.dimension())),
       m_C(theC),
       m_D(MatrixType::Identity(theX.dimension())),
       m_Bjac(MatrixType::Identity(theX.dimension())),
       m_R(theX.dimension(),theX.dimension()),
       m_Cjac(MatrixType::Identity(theX.dimension())),
       m_R0(theX.dimension(),theX.dimension()),
       m_Norm(theNorm.clone()),
       m_sizeFactor(sFactor)
{
   m_r.clear();
   m_R.clear();
   m_R0.clear();
}

//-----------------------------------------------

template<typename MatrixType>
void C1rect2<MatrixType>::move(capd::dynsys::C1DynSys<MatrixType> &c1dynsys, C1rect2 &result) const
{
   int dim = m_x.dimension();
   VectorType s(dim), Rem(dim);
   MatrixType S(dim,dim), Z(dim,dim),
              D1(dim,dim), Y(dim,dim), A(dim,dim),
              deltaA(dim,dim), jacEnc(dim,dim), jacRem(dim,dim);

   VectorType xx = m_x + m_C*m_r0 + m_B*m_r;
   VectorType y = c1dynsys.Phi(m_x);
   split(y,s);

   int found;
   VectorType Enc = c1dynsys.enclosure(xx,&found);
   jacEnc = c1dynsys.jacEnclosure(Enc,*m_Norm);
   c1dynsys.JacRemainder(Enc,jacEnc,Rem,jacRem);
   s += Rem;

   MatrixType Jac = c1dynsys.JacPhi(xx);
   result.m_C = Jac*m_C;
   splitMatrix(result.m_C,S);
   s += S*m_r0;
   A = Jac*m_B;
   MatrixType Q = midMatrix(A);
   orthonormalize(Q);
   MatrixType Qtr = Transpose(Q);
   result.m_x = y;
   result.m_r = (Qtr*A)*m_r + Qtr*s;
   result.m_B = Q;
   if(&result!=this) result.m_r0 = m_r0;
// T.Kapela - put set r into r0 if r is too big.
   if(m_sizeFactor!=0)
      if( ::size(result.m_r) > m_sizeFactor * ::size(result.m_r0))
      {
         result.m_r0 = result.m_r +  ((Qtr * result.m_C) * result.m_r0);
         result.m_C = result.m_B;
         result.m_B = MatrixType::Identity(dim);
         result.m_r = ScalarType(0.);
      }

//-----------------------------------------------

   A = Jac + jacRem;
   splitMatrix(A,deltaA);

   D1 = m_D + m_Bjac*m_R + m_Cjac*m_R0;
   result.m_Cjac = A * m_Cjac;
   splitMatrix(result.m_Cjac,S);

   Y = A*m_D + deltaA*D1;
   splitMatrix(Y,Z);
   Z += S*m_R0;
   result.m_D = Y;
   S = A*m_Bjac;
   Q = midMatrix(S);
   orthonormalize(Q);
   Qtr = Transpose(Q);
   result.m_R = (Qtr*S)*m_R + Qtr*Z;
   result.m_Bjac = Q;
   if (&result!=this) result.m_R0 = m_R0;

   if(m_sizeFactor!=0)
   {
      typename ScalarType::BoundType maxSizeR = 0., maxSizeR0 = 0.;
      int i,j;
      for(i=0;i<dim;i++)
         for(j=0;j<dim;j++)
         {
            maxSizeR = capd::max(maxSizeR,diam(result.m_R[i][j]).rightBound());
            maxSizeR0 = capd::max(maxSizeR0, diam(result.m_R0[i][j]).rightBound());
         }

      if(maxSizeR>m_sizeFactor*maxSizeR0)
      {
         result.m_R0 = result.m_R + (Qtr * result.m_Cjac) * result.m_R0;
         result.m_Cjac = result.m_Bjac;
         result.m_Bjac = MatrixType::Identity(dim);
         result.m_R.clear();
      }
   }
}

// ------------------------------------------------------------------------

template<typename MatrixType>
typename C1rect2<MatrixType>::ScalarType C1rect2<MatrixType>::size(void) const
{
   return ::size(m_x + m_C*m_r0 + m_B*m_r);
}

template<typename MatrixType>
std::string C1rect2<MatrixType>::show(void) const
{
   std::ostringstream descriptor;
   descriptor << "C1rect2:\n x=";
   descriptor << m_x << "\n B=";
   descriptor << m_B << "\n r=";
   descriptor << m_r << "\n C= ";
   descriptor << m_C << "\n r0= ";
   descriptor << m_r0 << "\n D= ";
   descriptor << m_D << "\n Bjac= ";
   descriptor << m_Bjac << "\n R= ";
   descriptor << m_R << "\n Cjac= ";
   descriptor << m_Cjac << "\n R0= ";
   descriptor << m_R0 << '\0';

   return descriptor.str();
}

template<typename MatrixT>
C1rect2<MatrixT>::operator typename C1rect2<MatrixT>::MatrixType() const
{
   return m_D + m_Bjac*m_R + m_Cjac*m_R0;
}

template<typename MatrixType>
C1rect2<MatrixType> &C1rect2<MatrixType>::operator=(const VectorType &theX)
{
   int dim = theX.dimension();
   m_x = theX;
   m_r0 = VectorType(dim);
   split(m_x,m_r0);
   m_C = MatrixType::Identity(dim);
   m_B = MatrixType::Identity(dim);
   m_r = VectorType(dim);
   m_r.clear();

   m_Bjac = m_Cjac = m_D = MatrixType::Identity(dim);
   m_R = MatrixType(dim,dim);
   m_R.clear();
   m_R0 = MatrixType(dim,dim);
   m_R0.clear();

   return *this;
}

template<typename MatrixType>
C1rect2<MatrixType>& C1rect2<MatrixType>::operator=(const C1rect2 &S)
{
   m_x = S.m_x;
   m_B = S.m_B;
   m_r = S.m_r;
   m_C = S.m_C;
   m_r0 = S.m_r0;

   m_D = S.m_D;
   m_Bjac = S.m_Bjac;
   m_R = S.m_R;
   m_Cjac = S.m_Cjac;
   m_R0 = S.m_R0;

   m_sizeFactor = S.m_sizeFactor;
   delete m_Norm;
   m_Norm = S.m_Norm->clone();
   return *this;
}

}} // namespace capd::dynset

#endif // _C1RECT_HPP

/// @}
