/// @addtogroup dynset
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file c1rectintf.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

/** Author: Daniel Wilczak, 2002-2004 */

#ifndef _C1rectIntf_H
#define _C1rectIntf_H

#include "dynset/c1set.h"
#include "vectalg/normintf.h"
#include "dynsys/c1dynsys.h"


/* C^1-Lohner algorithm.
   Derivative of the flow moved via QR decomposition (third method)
   the set part - QR decomposition (3-rd method)
*/

namespace capd{
namespace dynset{

template<typename MatrixT>
class C1rect : public C1set<MatrixT>{
public:
   typedef MatrixT MatrixType;
   typedef typename MatrixType::RowVectorType VectorType;
   typedef typename MatrixType::ScalarType ScalarType;
   typedef Norm<VectorType,MatrixType> NormType;

   C1rect(const NormType& theNorm, int dim);
   C1rect(const VectorType& theX, const NormType& theNorm);
   C1rect(const VectorType& theX, const VectorType& theR, const NormType& theNorm);
   C1rect(const VectorType& theX, const MatrixType&, const VectorType& theR, const NormType& theNorm);
   C1rect(const C1rect& c);
   ~C1rect(){delete m_Norm;}

   ScalarType size(void) const;
   C1set<MatrixType>* clone(void) const;
   C1set<MatrixType>* fresh(void) const;
   C1set<MatrixType>* cover(const VectorType& v) const;
   VectorType center(void) const;
   void move(capd::dynsys::C1DynSys<MatrixType>& c1dynsys);
   void move(capd::dynsys::C1DynSys<MatrixType>& c1dynsys, C1rect& result) const;

   std::string show(void) const;
   NormType *getNorm(void);
   operator VectorType() const;
   operator MatrixType() const;
   C1rect& operator=(const VectorType &v);
   C1rect& operator=(const C1rect &S);

protected:

// a representation of the set: x + B * r

   VectorType m_x,m_r;
   MatrixType m_B;

   MatrixType m_D, m_Bjac, m_R; // D + Bjac * R

   NormType *m_Norm; //the logarithmic norm used in the computation of a rough
         // enclosure for the variational part
};

// ------------------------------------------------

template<typename MatrixType>
inline void C1rect<MatrixType>::move(capd::dynsys::C1DynSys<MatrixType> &c1dynsys)
{
   move(c1dynsys,*this);
}

template<typename MatrixType>
inline C1set<MatrixType> * C1rect<MatrixType>::clone(void) const
{
   return new C1rect<MatrixType>(*this);
}

template<typename MatrixType>
inline C1set<MatrixType> *C1rect<MatrixType>::fresh(void) const
{
   return new C1rect<MatrixType>(*m_Norm,m_x.dimension());
}

template<typename MatrixType>
inline C1set<MatrixType> *C1rect<MatrixType>::cover(const VectorType &v) const
{
   return new C1rect(midVector(v),v-midVector(v), *m_Norm);
}

template<typename MatrixType>
inline typename C1rect<MatrixType>::VectorType C1rect<MatrixType>::center(void) const
{
   return m_x;
}

template<typename MatrixT>
inline C1rect<MatrixT>::operator typename C1rect<MatrixT>::VectorType(void) const
{
   return m_x + m_B*m_r;
}

template<typename MatrixType>
inline typename C1rect<MatrixType>::NormType *C1rect<MatrixType>::getNorm(void)
{
   return m_Norm;
}

}} // the end of the namespace capd::dynset

#endif //_C1rectIntf_H

/// @}
