/// @addtogroup poincare
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file basicCnTaylor.hpp
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

/** Author: Daniel Wilczak, 2006 */

#ifndef _BasicCnTaylor_HPP
#define _BasicCnTaylor_HPP

#include "dynsys/basicCnTaylorIntf.h"

namespace capd{
namespace dynsys{

template<typename CnMapT>
BasicCnTaylor<CnMapT>::~BasicCnTaylor(){}

template<typename CnMapT>
void BasicCnTaylor<CnMapT>::setOrder(int a_order)
{
   if(m_order==a_order+1) return;
   m_order = a_order+1;
   m_vectorField.setOrder(m_order);
   m_vectorFieldSeries.setOrder(a_order+2);
   m_resultSeries.setOrder(a_order+2);
   m_compositionFormula.setOrder(a_order+2);
}

// ---------------------------- CONSTRUCTORS ---------------------------------------------

template<typename CnMapT>
BasicCnTaylor<CnMapT>::BasicCnTaylor(const CnMapType& a_vectorField, int a_order, const ScalarType& a_step)
   : m_vectorField(a_vectorField), m_step(a_step), m_order(a_order+1), 
     m_vectorFieldSeries(a_vectorField.getDimension(),a_vectorField.getRank(),a_order+2),
     m_resultSeries(a_vectorField.getDimension(),a_vectorField.getRank(),a_order+2),
     m_compositionFormula(a_vectorField.getDimension(),a_vectorField.getRank(),true),
     m_nonlinearPart(a_vectorField.getDimension(),a_vectorField.getRank(),true),
     m_zeroNode(ScalarType(0),a_order+3)
{
   if(m_vectorField.getOrder()!=m_order)
      m_vectorField.setOrder(m_order);

   Multiindex::generateList(getDimension(),getRank(),m_listIndices);
   computeCompositionFormula();
}

// ---------------------------------------------------------------------------------------

template<typename CnMapT>
BasicCnTaylor<CnMapT>::BasicCnTaylor(const std::string& a_vectorField, int a_order, const ScalarType& a_step)
   : m_vectorField(a_vectorField,a_order+1), m_step(a_step), m_order(a_order+1),
     m_vectorFieldSeries(getDimension(),m_vectorField.getRank(),a_order+2),
     m_resultSeries(getDimension(),m_vectorField.getRank(),a_order+2),
     m_compositionFormula(getDimension(),m_vectorField.getRank(),true),
     m_nonlinearPart(getDimension(),m_vectorField.getRank(),true),
     m_zeroNode(ScalarType(0),a_order+3)

{
   Multiindex::generateList(getDimension(),getRank(),m_listIndices);
   computeCompositionFormula();
}

// ---------------------------------------------------------------------------------------

template<typename CnMapT>
BasicCnTaylor<CnMapT>::BasicCnTaylor(const char* a_vectorField, int a_order, const ScalarType& a_step)
   : m_vectorField(a_vectorField,a_order+1), m_step(a_step), m_order(a_order+1),
     m_vectorFieldSeries(getDimension(),m_vectorField.getRank(),a_order+2),
     m_resultSeries(getDimension(),m_vectorField.getRank(),a_order+2),
     m_compositionFormula(getDimension(),m_vectorField.getRank(),true),
     m_nonlinearPart(getDimension(),m_vectorField.getRank(),true),
     m_zeroNode(ScalarType(0),a_order+3)

{
   Multiindex::generateList(getDimension(),getRank(),m_listIndices);
   computeCompositionFormula();
}

// ---------------------------------------------------------------------------------------

template<typename CnMapT>
typename BasicCnTaylor<CnMapT>::NodeType& BasicCnTaylor<CnMapT>::computeProduct(const Multiindex& mi, const Multipointer& a, int p, int k)
{
   NodeType* result=0;
   bool first = true;
   const typename Multipointer::IndicesSet& is = Multipointer::generateList(p,k);
   typename Multipointer::IndicesSet::const_iterator b=is.begin(), e=is.end();
   while(b!=e)
   {
      typename Multipointer::MultipointersVector::const_iterator bt = b->begin(), et=b->end();
      typename Multiindex::const_iterator ib= mi.begin();

      Multipointer delta = a.subMultipointer(*bt);
      NodeType* temp = m_resultSeries(*ib,delta);
      ++bt;
      ++ib;
      while(bt!=et)
      {
         Multipointer delta = a.subMultipointer(*bt);
         temp = &( (*temp) * (*m_resultSeries(*ib,delta)) );
         ++bt;
         ++ib;
      }
      ++b;
      if(first)
      {
         result = temp;
         first = false;
      }else
         result = &( (*temp) + (*result) );
   }
   return *result;
}

// ---------------------------------------------------------------------------------------

template<typename CnMapT>
void BasicCnTaylor<CnMapT>::computeCompositionFormula(const Multipointer& a)
{
   int p = a.module(),i;
   capd::map::CnContainer<bool> mask(getDimension(),getRank());
   m_vectorField.getMask(mask);

   for(int k=2;k<=p;++k)
   {
      typename Multiindex::MultiindexVector::iterator b = m_listIndices[k-1].begin(), e=m_listIndices[k-1].end();
      while(b!=e)
      {
         Multipointer mp(b->dimension(),b->begin());
         std::sort(mp.begin(),mp.end());
         NodeType * product = 0;
         for(i=0;i<getDimension();++i)
         {
            if(!(product || mask(i,mp)) ) 
               product = &(computeProduct(*b,a,p,k));

            if(mask(i,mp)) continue;
            
            NodeType& temp = *(m_vectorFieldSeries(i,mp)) * (*product);
            if(m_nonlinearPart(i,a))
               m_nonlinearPart(i,a) = &( *(m_nonlinearPart(i,a)) + temp);
            else
               m_nonlinearPart(i,a) = &temp;
         }
         ++b;
      }
   }

   if(p>=2)
      for(i=0;i<getDimension();++i)
         if(!(m_nonlinearPart(i,a)))
         {
            m_nonlinearPart(i,a) = &m_zeroNode;
            m_zeroNode.m_links++;
         }
// for k=1
   typename Multiindex::MultiindexVector::iterator b = m_listIndices[0].begin(), e=m_listIndices[0].end();
   while(b!=e)
   {
      Multipointer mp(b->dimension(),b->begin());
      std::sort(mp.begin(),mp.end());
      NodeType* product = 0;

      for(i=0;i<getDimension();++i)
      {
         if(!(product || mask(i,mp)) ) 
            product = &(computeProduct(*b,a,p,1));

         if(mask(i,mp)) continue;

         NodeType& temp = *(m_vectorFieldSeries(i,mp)) * (*product);
         if(m_compositionFormula(i,a))
            m_compositionFormula(i,a) = &( *(m_compositionFormula(i,a)) + temp );
         else if(m_nonlinearPart(i,a))
         {
            m_compositionFormula(i,a) = &( *(m_nonlinearPart(i,a)) + temp );
            (m_nonlinearPart(i,a)->m_links)++;
         }
         else
            m_compositionFormula(i,a) = &temp;
      }
      ++b;
   }
   for(i=0;i<getDimension();++i)
      if(!(m_compositionFormula(i,a)))
      {
         m_compositionFormula(i,a) = &m_zeroNode;
         m_zeroNode.m_links++;
      }
 
   for(i=0;i<getDimension();++i)
      (m_compositionFormula(i,a)->m_links)++;
}

// ---------------------------------------------------------------------------------------

template<typename CnMapT>
void BasicCnTaylor<CnMapT>::computeCompositionFormula()
{
// for rank==0
   for(int j=0;j<getDimension();++j)
   {
      m_compositionFormula(j) = m_vectorFieldSeries(j);
      (m_compositionFormula(j)->m_links)++;
   }

   for(int i=1;i<=getRank();++i)
   {
      Multipointer a = m_compositionFormula.first(i);
      do{
         computeCompositionFormula(a);
      }while(m_compositionFormula.next(a));
   }
}

// ---------------------------------------------------------------------------------------

template<typename CnMapT>
typename BasicCnTaylor<CnMapT>::VectorType BasicCnTaylor<CnMapT>::operator()(CnCoeffType& coeff) const
{
   if(getDimension()!=coeff.dimension())
      throw std::runtime_error("BasicCnTaylor::operator(CnCoeffType&) - incompatible dimensions");
   int rank = coeff.rank();
   if( getRank() < rank )
      throw std::runtime_error("BasicCnTaylor::operator(CnCoeffType&) - requested rank to large");

   setInitialCondition(coeff);
   computeVectorFieldSeries(VectorType(coeff),rank,m_order);
   computeResultSeries(rank,m_order);

   coeff.clear();
   for(int j=m_order-1;j>=0;--j)
   {
      for(int i=0;i<getDimension();++i)
      {
         typename SeriesContainerType::const_iterator it=m_resultSeries.begin(i);
         typename CnCoeffType::iterator b = coeff.begin(i), e=coeff.end(i,rank);
         while(b!=e)
         {
            (*b) = (*b) * m_step + (*it)->value[j];
            ++b;
            ++it;
         }      
      }
   }

   return VectorType(coeff);
}

// ---------------------------------------------------------------------------------------

template<typename CnMapT>
void BasicCnTaylor<CnMapT>::setInitialCondition(const CnCoeffType& coeff) const
{
   for(int i=0;i<getDimension();++i)
   {
      typename CnCoeffType::const_iterator b=coeff.begin(i), e=coeff.end(i,coeff.rank());
      typename SeriesContainerType::const_iterator j=m_resultSeries.begin(i), p = m_compositionFormula.begin(i);
      while(b!=e)
      {
         (*j)->value[0] = *b;
         (*p)->reset();
         ++b;
         ++j;
         ++p;
      }
   }
}

// ---------------------------------------------------------------------------------------

template<typename CnMapT>
void BasicCnTaylor<CnMapT>::setInitialCondition(const VectorType& v, const MatrixType& m) const
{
   int i,j;
   for(i=0;i<getDimension();++i)
   {
      m_resultSeries(i)->value[0] = v[i];
      for(j=0;j<getDimension();++j)   
         m_resultSeries(i,j)->value[0] = m(i+1,j+1);
   }
   typename SeriesContainerType::const_iterator b,e;
   for(j=2;j<=getRank();++j)
      for(i=0;i<getDimension();++i)
      {
         b=m_resultSeries.begin(i,j);
         e=m_resultSeries.end(i,j);
         while(b!=e)
         {
            (*b)->value[0]=ScalarType(0);
            ++b;
         }
      }

   b=m_compositionFormula.begin();
   e=m_compositionFormula.end();
   while(b!=e)
   {
      (*b)->reset();
      ++b;
   }
}

// ---------------------------------------------------------------------------------------

template<typename CnMapT>
void BasicCnTaylor<CnMapT>::setInitialCondition(const VectorType& v, const MatrixType& m, const C2CoeffType& hessian) const
{
   int i,j,c;
   for(i=0;i<getDimension();++i)
   {
      m_resultSeries(i)->value[0] = v[i];
      for(j=0;j<getDimension();++j)
      {
         m_resultSeries(i,j)->value[0] = m(i+1,j+1);
         for(c=j;c<getDimension();++c)
            m_resultSeries(i,j,c)->value[0] = hessian(i,j,c);
      }
   }
   typename SeriesContainerType::const_iterator b,e;
   for(j=3;j<=getRank();++j)
      for(i=0;i<getDimension();++i)
      {
         b=m_resultSeries.begin(i,j);
         e=m_resultSeries.end(i,j);
         while(b!=e)
         {
            (*b)->value[0]=ScalarType(0);
            ++b;
         }
      }

   b=m_compositionFormula.begin();
   e=m_compositionFormula.end();
   while(b!=e)
   {
      (*b)->reset();
      ++b;
   }
}

// ---------------------------------------------------------------------------------------

template<typename CnMapT>
void BasicCnTaylor<CnMapT>::setInitialCondition(const VectorType& v) const
{
   for(int i=0;i<getDimension();++i)
      m_resultSeries(i)->value[0] = v[i];

   typename SeriesContainerType::const_iterator b=m_compositionFormula.begin(), e=m_compositionFormula.end();
   while(b!=e)
   {
      (*b)->reset();
      ++b;
   }
}


// ---------------------------------------------------------------------------------------

template<typename CnMapT>
void BasicCnTaylor<CnMapT>::computeVectorFieldSeries(const VectorType& v, int rank, int a_order) const
{
   VectorType x = v;
   for(int j=0;j<a_order;++j)
   {
      m_vectorField.computeDerivatives(x,m_vectorFieldSeries,j,0,rank);
      for(int i=0;i<getDimension();++i)
         x[i] = m_resultSeries(i)->value[j+1] = m_vectorFieldSeries(i)->value[j]/(j+1);
   }
}


// ---------------------------------------------------------------------------------------

template<typename CnMapT>
void BasicCnTaylor<CnMapT>::computeResultSeries(int rank, int a_order) const
{
   for(int p=0;p<a_order;++p)
      for(int r=1;r<=rank;++r)
      {
         for(int i=0;i<getDimension();++i)
         {
            typename SeriesContainerType::const_iterator 
               b = m_compositionFormula.begin(i,r), 
               e = m_compositionFormula.end(i,r),
               j = m_resultSeries.begin(i,r);
            while(b!=e)
            {
               (*j)->value[p+1] = (*b)->operator()(p)/(p+1);
               ++b;
               ++j;
            }
         }
      }
}

// ---------------------------------------------------------------------------------------

template<typename CnMapT>
void BasicCnTaylor<CnMapT>::computeNonlinearPart(int rank) const
{
   for(int i=0;i<getDimension();++i)
   {
      typename SeriesContainerType::const_iterator 
         b = m_nonlinearPart.begin(i,rank), 
         e = m_nonlinearPart.end(i,rank);
      while(b!=e)
      {
         (*b)->operator()(0);
         ++b;
      }      
   }
}

// ---------------------------------------------------------------------------------------

template<typename CnMapT>
typename BasicCnTaylor<CnMapT>::VectorType BasicCnTaylor<CnMapT>::operator()(const VectorType& iv) const
{
   computeVectorFieldSeries(iv,0,m_order);
   setInitialCondition(iv);
   VectorType result(getDimension());
   int i,j;
   for(i=0;i<getDimension();++i)
      result[i] = m_resultSeries(i)->value[m_order];

   for(j=m_order-1;j>=0;--j)
   {
      for(i=0;i<getDimension();++i)
         result[i] = result[i]*m_step + m_resultSeries(i)->value[j];
   }
   return result;
}

// ---------------------------------------------------------------------------------------

template<typename MapType>
typename BasicCnTaylor<MapType>::VectorType BasicCnTaylor<MapType>::operator()(const VectorType &iv, MatrixType& result) const
{
   setInitialCondition(iv,MatrixType::Identity(getDimension()));
   computeVectorFieldSeries(iv,1,m_order);
   computeResultSeries(1,m_order);
   VectorType x(getDimension());

   int i,j,c;
   for(i=0;i<getDimension();++i)
   {
      x[i] = m_resultSeries(i)->value[m_order];
      for(j=0;j<getDimension();++j)
         result(i+1,j+1) = m_resultSeries(i,j)->value[m_order];
   }
   for(c=m_order-1;c>=0;--c)
      for(i=0;i<getDimension();++i)
      {
         x[i] = x[i]*m_step + m_resultSeries(i)->value[c];
         for(j=0;j<getDimension();++j)
            result(i+1,j+1) = result(i+1,j+1)*m_step + m_resultSeries(i,j)->value[c];
      }
   return x;
}


// ---------------------------------------------------------------------------------------

template<typename MapType>
const typename BasicCnTaylor<MapType>::SeriesContainerType& 
   BasicCnTaylor<MapType>::computeDaPhi(const CnCoeffType& c, int order) const
{
   if(order>m_order)
      throw std::runtime_error("BasicCnTaylor::computeComposition - requested order to large");
   setInitialCondition(c);
   computeVectorFieldSeries(VectorType(c),c.rank(),order);
   computeResultSeries(c.rank(),order);
   return m_resultSeries;
}

}} // the end of the namespace capd::dynsys

#endif // _BasicCnTaylor_HPP

/// @}
