/// @addtogroup dynsys
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file basicTaylor.hpp
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

/** Author: Daniel Wilczak, 2001-2004 */

#ifndef _BASICTAYLOR_HPP
#define _BASICTAYLOR_HPP

#include <sstream>
#include <string>
#include <stdexcept>

#include "capd/factrial.h"
#include "dynsys/basicTaylorIntf.h"

namespace capd{
namespace dynsys{

//###########################################################//

template<typename MapType>
BasicTaylor<MapType>::BasicTaylor(
      const MapType& vectorField,
      int a_order,
      const ScalarType& Step
   ) : m_vField(vectorField), m_order(a_order), m_step(Step)
{
   m_dimension = m_vField.getDimension();
   m_vField.setOrder(m_order+1);
   m_coeff = new (m_dimension) VectorType[m_order+2];
   m_matrixCoeff = new (m_dimension,m_dimension) MatrixType[m_order+2];
   m_F = new (m_dimension,m_dimension) MatrixType[m_order+2];
}

//###########################################################//

template<typename MapType>
BasicTaylor<MapType>::BasicTaylor(
      const std::string &vectorField,
      int a_order,
      const ScalarType &Step
   ) : m_vField(vectorField,a_order+1), m_order(a_order), m_step(Step)
{
   m_dimension = m_vField.getDimension();
   m_coeff = new (m_dimension) VectorType[m_order+2];
   m_matrixCoeff = new (m_dimension,m_dimension) MatrixType[m_order+2];
   m_F = new (m_dimension,m_dimension) MatrixType[m_order+2];
}

//###########################################################//


template<typename MapType>
BasicTaylor<MapType>::BasicTaylor(
      const char* vectorField,
      int a_order,
      const ScalarType& Step
   ) : m_vField(vectorField,a_order+1), m_order(a_order), m_step(Step)
{
   m_dimension = m_vField.getDimension();
   m_coeff = new (m_dimension) VectorType[m_order+2];
   m_matrixCoeff = new (m_dimension,m_dimension) MatrixType[m_order+2];
   m_F = new (m_dimension,m_dimension) MatrixType[m_order+2];
}


//###########################################################//

template<typename MapType>
BasicTaylor<MapType>& BasicTaylor<MapType>::operator=(const BasicTaylor& a_t) 
{
   m_vField = a_t.m_vField;
   m_order = a_t.m_order;
   m_step = a_t.m_step;
   m_dimension = a_t.m_dimension;
   m_coeff = new (m_dimension) VectorType[m_order+2];
   m_matrixCoeff = new (m_dimension,m_dimension) MatrixType[m_order+2];
   m_F = new (m_dimension,m_dimension) MatrixType[m_order+2];

   for(int i=0; i<m_order+2; ++i)
   {
      m_coeff[i] = a_t.m_coeff[i];
      m_matrixCoeff[i] = a_t.m_matrixCoeff[i];
      m_F[i] = a_t.m_F[i];
   }
   return *this;
}

//###########################################################//

template<typename MapType>
BasicTaylor<MapType>::BasicTaylor(
      const BasicTaylor &t
   ) : m_vField(t.m_vField), m_order(t.m_order), m_step(t.m_step),  m_dimension(t.m_dimension)
{

   m_coeff = new (m_dimension) VectorType[m_order+2];
   m_matrixCoeff = new (m_dimension,m_dimension) MatrixType[m_order+2];
   m_F = new (m_dimension,m_dimension) MatrixType[m_order+2];

   for(int i=0; i<m_order+2; ++i)
   {
      m_coeff[i] = t.m_coeff[i];
      m_matrixCoeff[i] = t.m_matrixCoeff[i];
      m_F[i] = t.m_F[i];
   }
}

//###########################################################//


template<typename MapType>
BasicTaylor<MapType>::~BasicTaylor()
{
   delete [] m_coeff;
   delete [] m_matrixCoeff;
   delete [] m_F;
}

//###########################################################//


template<typename MapType>
void BasicTaylor<MapType>::setOrder(int newOrder)
{
   if(newOrder!=m_order)
   {
      m_order = newOrder;
      m_vField.setOrder(m_order+1);

      delete [] m_coeff;
      delete [] m_matrixCoeff;
      delete [] m_F;
      m_coeff = new (m_dimension) VectorType[m_order+2];
      m_matrixCoeff = new (m_dimension,m_dimension) MatrixType[m_order+2];
      m_F = new (m_dimension,m_dimension) MatrixType[m_order+2];
   }
}


//###########################################################//


template<typename MapType>
void BasicTaylor<MapType>::setField(const MapType& vectorField)
{
   m_vField = vectorField;
   m_vField.setOrder(m_order+1);
   if(m_dimension!=m_vField.getDimension())
   {
      delete [] m_coeff;
      delete [] m_matrixCoeff;
      delete [] m_F;
      m_dimension = m_vField.getDimension();
      m_coeff = new (m_dimension) VectorType[m_order+2];
      m_matrixCoeff = new (m_dimension,m_dimension) MatrixType[m_order+2];
      m_F = new (m_dimension,m_dimension) MatrixType[m_order+2];
   }
}


//###########################################################//


template<typename MapType>
void BasicTaylor<MapType>::computeMatrixCoeff(int ile, const MatrixType &V) const
{
   if(ile<0)
   {
      std::ostringstream message;
      message << "Taylor error: negative index " << ile << " of Taylor coefficient in function computeMatrixCoeff";
      throw std::out_of_range(message.str());
   }

   m_matrixCoeff[0] = V;

   int p,i,j,k,r;
   for(p=0;p<=ile;++p)
   {
      typename MatrixType::ContainerIterator b = m_matrixCoeff[p+1].cbegin();
      for(i=1;i<=m_dimension;++i)
         for(j=1;j<=m_dimension;++j)
         {
            ScalarType result(0);
            for(k=1;k<=m_dimension;++k)
               for(r=0;r<=p;++r)
                  result += m_F[r](i,k) * m_matrixCoeff[p-r](k,j);
            *b = result/(p+1);
            ++b;
         }
   }
}


//###########################################################//


template<typename MapType>
typename BasicTaylor<MapType>::VectorType BasicTaylor<MapType>::operator()(const VectorType &iv) const
{
   m_coeff[0] = iv;   //  coeff will contain  Taylor coefficients in <iv>
                      // now we initiate 0-th order term

   int r;
   for(r=0;r<m_order;++r)
      m_coeff[r+1] = m_vField(m_coeff[r],r)/(r+1);  // recurrent computation of consequtive
                                             // Taylor coefficients
   VectorType result = m_coeff[m_order];
   for(r=m_order-1;r>=0;--r)
      result = result*m_step + m_coeff[r];
   return result;
}


//###########################################################//


template<typename MapType>
typename BasicTaylor<MapType>::VectorType BasicTaylor<MapType>::operator()(
      const VectorType &iv, MatrixType& der
   ) const
{
  // first we compute ceofficients for x
   int r;
   m_coeff[0] = iv;
   for(r=0;r<m_order;++r)
      m_coeff[r+1] = m_vField(m_coeff[r],r)/(r+1);

   VectorType result = m_coeff[m_order];
   for(r=m_order-1;r>=0;--r)
      result = result*m_step + m_coeff[r];

  // now we compute the coefficients for the Jacobian matrix
  // the coefficients will be stored in F

   m_vField.variational(m_coeff,m_F,m_order);
   // this function compute coefficients up to index 'order'
   computeMatrixCoeff(m_order-1,MatrixType::Identity(m_dimension));

  // the sumation of the Taylor series
   der = m_matrixCoeff[m_order];
   for(r=m_order-1;r>=0;--r)
      der = der*m_step + m_matrixCoeff[r];

   return result;
}


}} //namespace capd::dynsys

#endif // #define _BASICTAYLOR_HPP

/// @}
