/// @addtogroup dynsys
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file cntaylintf.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

/** Author: Daniel Wilczak, 2006 */

#ifndef _CnTaylIntf_H
#define _CnTaylIntf_H

#include <string>
#include <vector>
#include "vectalg/normintf.h"
#include "dynsys/c2taylintf.h"
#include "dynsys/cndynsys.h"
#include "dynsys/basicCnTaylorIntf.h"

namespace capd{
namespace dynsys{

template<typename MapT>
class CnTaylor : public BasicCnTaylor<MapT>, public CnDynSys<typename MapT::MatrixType>{
public:
   typedef MapT MapType;
   typedef MapT CnMapType;
   typedef typename MapType::FunctionType FunctionType;
   typedef typename CnMapType::MatrixType MatrixType;
   typedef typename MatrixType::RowVectorType VectorType;
   typedef typename MatrixType::ScalarType ScalarType;
   typedef typename CnMapType::C2CoeffType C2CoeffType;
   typedef typename CnMapType::CnCoeffType CnCoeffType;
   typedef typename CnMapType::NodeType NodeType;
   typedef typename CnMapType::ConsNodeType ConsNodeType;
   typedef typename CnMapType::SeriesContainerType SeriesContainerType;
   typedef typename SeriesContainerType::Multipointer Multipointer;
   typedef typename SeriesContainerType::Multiindex Multiindex;
   typedef Norm<VectorType,MatrixType> NormType;

   CnTaylor(const MapType& vectorField, int Order, const ScalarType& Step);
   CnTaylor(const std::string& vectorField, int Order, const ScalarType& Step);
   CnTaylor(const char* vectorField, int Order, const ScalarType& Step);
//   CnTaylor& operator=(const C2Taylor&);

// enclosure
   VectorType enclosure(const VectorType& x, int* found) const;
   VectorType enclosure(const VectorType& x) const;
   MatrixType jacEnclosure(const VectorType& enc, NormType& norm) const;
   void c2Enclosure(
         const VectorType& W1,
         const MatrixType& W3,
         NormType& the_norm,
         C2CoeffType& result
   ) const;
   void cnEnclosure(const VectorType& x, CnCoeffType& result, NormType& the_norm) const;
   VectorType cnEnclosure(const VectorType& x, NormType& the_norm, int rank) const;

// Phi
   VectorType Phi(const VectorType &iv) const;
   MatrixType JacPhi(const VectorType &iv) const;
   void c2Phi(const VectorType& x, MatrixType& jac, C2CoeffType& hessian) const;
   void cnPhi(const VectorType&x, CnCoeffType& result) const;

// remainder
   VectorType Remainder(const VectorType &iv) const;
   void JacRemainder(
            const VectorType &vecEnclosure,
            const MatrixType &jacEnclosure,
            VectorType &Remainder,
            MatrixType &jRemainder
      ) const;
   void c2Remainder(
         const VectorType& Enclosure,
         const MatrixType& jacEnclosure,
         const C2CoeffType& hessianEnclosure,
         VectorType& Remainder,
         MatrixType& jacRemainder,
         C2CoeffType& hessianRemainder // here a result is stored
   ) const;
   void cnRemainder(const VectorType& x, NormType&, CnCoeffType& result) const;

   using DynSys<MatrixType>::Lipschitz;
   using DynSys<MatrixType>::type;
   using BasicCnTaylor<MapType>::getDimension;
   using BasicCnTaylor<MapType>::getRank;

protected:
   using BasicCnTaylor<MapType>::computeNonlinearPart;
   using BasicCnTaylor<MapType>::m_order;
   using BasicCnTaylor<MapType>::m_step;
   using BasicCnTaylor<MapType>::m_vectorField;
   using BasicCnTaylor<MapType>::m_vectorFieldSeries;
   using BasicCnTaylor<MapType>::m_resultSeries;
   using BasicCnTaylor<MapType>::m_compositionFormula;
   using BasicCnTaylor<MapType>::m_nonlinearPart;
   using BasicCnTaylor<MapType>::computeVectorFieldSeries;
   using BasicCnTaylor<MapType>::computeResultSeries;
   
   ScalarType computeDelta(const Multipointer&) const;
   mutable typename ScalarType::BoundType m_l;
}; // the end of class CnTaylor

// ---------------------------- CONSTRUCTORS ---------------------------------------------

template<typename CnMapT>
inline CnTaylor<CnMapT>::CnTaylor(const CnMapType& a_vectorField, int a_order, const ScalarType& a_step)
   : BasicCnTaylor<CnMapT>(a_vectorField,a_order,a_step)
{}

// ---------------------------------------------------------------------------------------

template<typename CnMapT>
inline CnTaylor<CnMapT>::CnTaylor(const std::string& a_vectorField, int a_order, const ScalarType& a_step)
   : BasicCnTaylor<CnMapT>(a_vectorField,a_order,a_step)
{}

// ---------------------------------------------------------------------------------------

template<typename CnMapT>
inline CnTaylor<CnMapT>::CnTaylor(const char* a_vectorField, int a_order, const ScalarType& a_step)
   : BasicCnTaylor<CnMapT>(a_vectorField,a_order,a_step)
{}


template<typename MapType>
inline typename CnTaylor<MapType>::VectorType CnTaylor<MapType>::enclosure(const VectorType &x, int *found) const
// the function finds an enclosure for \varphi([0,step],x)
{
  return enclosure(x);
}

}} // the end of the namespace capd::dynsys

#endif // _CnTaylIntf_H

/// @}
