/// @addtogroup map
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file BasicFunctionIntf.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

/** Author: Daniel Wilczak, 2006 */

#ifndef _BasicFunctionIntf_H
#define _BasicFunctionIntf_H

#include <string>
#include <map>
#include "map/nodeintf.h"

// This class is a basic for further protected inheritance to classes Function, CnMap  ...
// It should not be use as a independent class

namespace capd{
namespace map{

/* ________________________ CLASS BASICFUNCTION _______________________*/

template<typename Scalar>
class BasicFunction
{
protected:
   typedef Scalar ScalarType;
   typedef Node<ScalarType> NodeType;

   BasicFunction();
   BasicFunction(const BasicFunction&f);
   BasicFunction(const std::string& s, int i);
   BasicFunction(const char* s, int i);
   virtual ~BasicFunction();
   
   void operator=(const BasicFunction&);
   void operator=(const std::string&);
   void operator=(const char* );
   
   int dimension(void) const {return m_dim;}
   void setOrder(int);
   void setParameter(const std::string &name, const ScalarType& value);
   std::string variables() const;
   
   bool depends(std::string, int i) const;
   bool isVariable(std::string&,int i) const;
   void eqnanal(std::string&, NodeType **);
   std::string diffanal(std::string&, int i) const;

   void createFromText(const std::string& s, int i);
   void createFromObject(const BasicFunction& s);
   void createDefault();
   void deleteTables();
   virtual const std::string& currentFormula()=0;

   std::string* m_var;   /* the table of variables */
   ScalarType* m_val;    /* the table of the values of coordinates */

   int m_dim;   /* dimension of domain */
   int m_order;
   int m_size;  /* size = dim*order */
   std::map<std::string, NodeType*> m_nodes;
};

// ------------------------- inline definitions ---------------------

template<typename Scalar>
inline void BasicFunction<Scalar>::operator=(const BasicFunction& f)
{
   if(this!=&f)
   {
      deleteTables();
      createFromObject(f);
   }
}

// -----------------------------------------------------------------------------

template<typename Scalar>
inline void BasicFunction<Scalar>::operator=(const std::string& s)
{
   deleteTables();
   createFromText(s,0);
}

// -----------------------------------------------------------------------------

template<typename Scalar>
inline void BasicFunction<Scalar>::operator=(const char* s)
{
   deleteTables();
   createFromText(s,0);
}

// -----------------------------------------------------------------------------

template<typename Scalar>
inline BasicFunction<Scalar>::BasicFunction()
{
   createDefault();
}

// -----------------------------------------------------------------------------

template<typename Scalar>
inline BasicFunction<Scalar>::BasicFunction(const BasicFunction& f)
{
   createFromObject(f);
}

// -----------------------------------------------------------------------------

template<typename Scalar>
inline BasicFunction<Scalar>::BasicFunction(const std::string&s, int a_order)
{
   createFromText(s,a_order);
}

// -----------------------------------------------------------------------------

template<typename Scalar>
inline BasicFunction<Scalar>::BasicFunction(const char* s, int a_order)
{
   createFromText(s,a_order);
}

// -----------------------------------------------------------------------------

template<typename Scalar>
inline BasicFunction<Scalar>::~BasicFunction()
{
   deleteTables();
}

}} // the end of the namespace capd::map

#endif //define _BasicFunction_H

/// @}
