/// @addtogroup map
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file c2coeff.hpp
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

/** Author: Daniel Wilczak,  2006 */

#ifndef _C2COEFF_HPP
#define _C2COEFF_HPP

namespace capd{
namespace map{

template<typename ScalarType>
int C2Coeff<ScalarType>::defaultDim = 0;

// ---------------------------------------------------

template<typename Scalar>
C2Coeff<Scalar>::C2Coeff(const C2Coeff& s)
{
   dim = s.dim;
   data = new ScalarType[dim*dim*(1+dim)/2];
   iterator i1=begin(), i2=end();
   const_iterator j1=s.begin();
   while(i1!=i2)
   {
      *i1=*j1;
      ++i1;
      ++j1;
   }
}

// ---------------------------------------------------

template <typename ScalarType>
C2Coeff<ScalarType>& C2Coeff<ScalarType>::operator=(const C2Coeff& s)
{
   if(dim!=s.dim)
   {
      delete []data;
      dim = s.dim;
      data = new ScalarType[dim*dim*(1+dim)/2];
   }
   iterator i1=begin(), i2=end();
   const_iterator j1 = s.begin();
   while(i1!=i2)
   {
      *i1=*j1;
      ++i1;
      ++j1;
   }
   return *this;
}

// ---------------------------------------------------

template <typename ScalarType,int d>
C2Coeff<ScalarType> operator*(const Matrix<ScalarType,d,d>& m, const C2Coeff<ScalarType>& c2)
{
   int dim = m.numberOfRows();
   if(dim!=c2.dimension())
   {
      throw std::runtime_error("operator* - incompatible dimensions of matrix and C2Coeff");
   }
   C2Coeff<ScalarType> result(dim);
   result.clear();
   for(int j=0;j<dim;++j)
      for(int c=j;c<dim;++c)
         for(int i=0;i<dim;++i)
            for(int r=0;r<dim;++r)
               result(i,j,c) += m[i][r]*c2(r,j,c);
   return result;
}


// ---------------------------------------------------

template <typename ScalarType>
C2Coeff<ScalarType>& C2Coeff<ScalarType>::operator+=(const C2Coeff& s)
{
   if(dim!=s.dim)
   {
      throw std::runtime_error("incompatible dimensions in C2Coeff::operator+=");
   }
   iterator i1=begin(), i2=end();
   const_iterator j1 = s.begin();
   while(i1!=i2)
   {
      *i1 += *j1;
      ++i1;
      ++j1;
   }
   return *this;
}

// ---------------------------------------------------

template <typename ScalarType>
void C2Coeff<ScalarType>::clear()
{
   iterator b=begin(), e=end();
   while(b!=e)
   {
      *b = ScalarType(0.);
      ++b;
   }
}

}}

#endif

/// @}
