/// @addtogroup newton
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file krawczyk.h
///
/// @author Tomasz Kapela
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2006 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.


#ifndef _NEWTON_H_
#define _NEWTON_H_

#include <string>
#include <sstream>
#include "newton/newtonResult.h"

namespace capd{
namespace newton{

/////////////////////////////////////////////////////////////////////////////////
///
/// Default function for writing details on Newton Proof.
/// You can define your own specification  to replace this default function
/////////////////////////////////////////////////////////////////////////////////

template <typename IntervalVector, typename IntervalMatrix>
inline void  NewtonInfo(int iter,   IntervalVector x0,   IntervalVector X,
                          IntervalVector F_x0,  IntervalMatrix dF_X,  IntervalVector K)
{

}

//*****************************************************************************/
// NewtonProof
///
///  Rigorous existence proof of a zero of a given function F
///   
///  We compute rigorously Newton operator \f$ N = x - (dF(X)^-1) F(x) \f$
///  and we check assumptions of the Interval Newton Method.
///
///   @param[in]  x      approximated zero of F,
///   @param[in]  size   radius of set X,  ( \f$X = x + [-size, size]^n \f$ )
///   @param[in]  X      set X
///   @param[in]  F      class which can calculate value of the function F and its derivative dF
///
///  @returns    NewtonResult constant:
///     zeroExists = 1         - zero exists (K is subset of X),
///     noZeroes = 0           - there is no zeros in the set X (intersection of K and X is empty),
///     tooManyIterations = -1 - we can still take intersection of X and K, but we exceed maximal number of iteration
///     undefined = -2         - X is subset of K,
///
/*****************************************************************************/

template <typename FloatVector, typename MapType>
NewtonResult NewtonProof ( FloatVector &x,  
                           double size,  
			   MapType &F) ;


template <typename MapType>
NewtonResult NewtonProof ( const typename MapType::VectorType& x0, 
                           const typename MapType::VectorType& X,
                           MapType &F);


///////////////////////////////////////////////////////////////////////////////////////
//  NewtonOperator
///
///   Computes Newton operator
///
///   @param[in]    x    approximated zero of F
///   @param[in]    X    set X
///   @param[in]    F    class which can calculate value of the function F and its derivative dF
///
///   @returns    N = x - (dF(X)^-1)*F(x)
///
///////////////////////////////////////////////////////////////////////////////////////

template <typename MapType>
typename MapType::VectorType NewtonOperator ( const typename MapType::VectorType& x0,  
                                              const typename MapType::VectorType& X, 
					      MapType &F);

}}  //end of namespace capd::newton

#include "newton/newton.hpp"

#endif //_NEWTON_H_

/// @}
