/// @addtogroup vecttst
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file vecttst.cpp
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#include <fstream>
#include <stdexcept>
#include "vectalg/vector.h"
#include "vectalg/matrix.h"

typedef Vector<double,3> vector;
typedef Matrix<double,3,3> matrix;
typedef Vector<interval,3> intervalVector;
typedef Matrix<interval,3,3> intervalMatrix;

void test_gauss(void)
{
   std::ifstream inp("matrix.txt");
   if(!inp)
      throw std::runtime_error("Failed to open inp");

   std::cout << "\n\n============================\n";
   std::cout << "Gauss test\n";
   std::cout << "============================\n\n";

   vector v,r;
   matrix a,b;
   intervalVector iv,ir;
   intervalMatrix ia;

   inp >> v;
   inp >> a >> b;

   iv = intervalVector(v);
   ia = intervalMatrix(a);

   r = gauss(a,v);

   round_up();
   ir = gauss(ia,iv);
   round_nearest();

   std::cout << "v=" << v << " iv=" << iv << std::endl;
   std::cout << "a=" << a << " ia=" << ia << std::endl;
   std::cout << "r=" << r << std::endl;
   std::cout << "ir=" << ir << std::endl;
   for(int i=0;i<ir.Dimension();i++)
      std::cout << diam(ir[i]) << std::endl;
}

void QRdecomp_test(void)
{
   std::cout << "\n\n============================\n";
   std::cout << "QR decompose test\n";
   std::cout << "============================\n\n";

   std::ifstream inp("qrdane.txt");
   if(!inp)
      throw std::runtime_error("Failed to open inp\n");

   matrix a;
   intervalMatrix ia,iq,ir;
   inp >> a;
   ia = intervalMatrix(a);

   std::cout << "ia=" << ia << "\n";

   QR_decompose(ia,iq,ir);
   for(int i=0;i<iq.numberOfRows();i++)
      for(int j=0;j<iq.numberOfColumns();j++)
      {
         interval sp=iq[i]*iq[j];
         std::cout << sp << std::endl;
      }
}

void diagonalize_test(void)
{
   rootFrame.Clear();
   rootFrame << "diagonalize test\n";
   std::ifstream inp("../tests/vecttst/diagdane.txt", std::ios::in);
   if(!inp) rootFrame << "Failed to open inp\n";

   matrix a;
   intervalMatrix ia,id;

   inp >> a;

   ia = intervalMatrix(a);

   rootFrame << "a=" << a << "\n";
   rootFrame << "ia=" << ia << "\n";

   diagonalize(ia,id);
   for(int i=0;i<DIM;i++)
   {
   rootFrame << At(5+i,0) << "id[" << i << "]=" << id[i] << "\n";
   }
   rootFrame << At(11,0) << "Spectral radius= " << spectralRadius(ia) << "\n";
   waitBt();
}

void norm_test(void)
{
   rootFrame.Clear();
   rootFrame << "norm test\n";
   std::ifstream inp("../tests/vecttst/normdane.txt");
   if(!inp) rootFrame << "Failed to open inp\n";

   matrix a;
   intervalMatrix ia;

   inp >> a;

   ia = intervalMatrix(a);

   rootFrame << "a=" << a << "\n";
   rootFrame << "ia=" << ia << "\n";

   euclNorm en;
   maxNorm mn;
   sumNorm sn;
   euclLNorm eln;
   maxLNorm mln;
   sumLNorm sln;

   rootFrame << "Euclidean norm = " << en(ia) << " \n";
   rootFrame << "Maximum norm = " << mn(ia) << " \n";
   rootFrame << "Sum norm = " << sn(ia) << " \n";
   rootFrame << "Euclidean logarithmic norm = " << eln(ia) << " \n";
   rootFrame << "Maximum logarithmic norm = " << mln(ia) << " \n";
   rootFrame << "Sum logarithmic norm = " << sln(ia) << " \n";

   waitBt();
}


void matrix_test(void)
{
   rootFrame.Clear();
   rootFrame << "matrix test\n";
   std::ifstream inp("../tests/vecttst/matrix.txt");
   if(!inp) rootFrame << "Failed to open inp\n";

   vector v;
   matrix a,b;
   intervalVector iv;
   intervalMatrix ia;

   inp >> v;
   inp >> a >> b;

   iv = intervalVector(v);
   ia = intervalMatrix(a);

   rootFrame << At(2,0) << "v=" << v << " iv=" << iv;
   rootFrame << At(3,0) << "a=" << a << " ia=" << ia;
   for(int i=0;i<DIM;i++)
   {
   rootFrame << At(6+i,0) << "ia[" << i << "]=" << ia[i] << "\n";
   }
   waitBt();
}


void ztest_gauss(void)
{
   rootFrame.Clear();
   rootFrame << "(z) Gauss test\n";

   intervalVector iv,r;
   intervalMatrix ia;

   /*
   iv[0]=interval(-7.45037e-16,-5.4116e-16);
   iv[1]=interval(-2.12588e-15,-9.94006e-16);
   iv[2]=interval(-4.44089e-21,4.44089e-21);
   */
   iv[0]=5.0;

   ia[0][0]=-0.237304;
   ia[0][1]= -1.18286;
   ia[0][2]=8.49793e-322;

   ia[1][0]=1.18286;
   ia[1][1]=-0.000731345;
   ia[1][2]=-5.7114e-321;

   ia[2][0]=-6.81317e-321;
   ia[2][1]=-1.34386e-321;
   ia[2][2]=1.0;

   r = gauss(ia,iv);

   intervalVector res;
   res=ia*r - iv;

   rootFrame << "ia*r - iv = " << res << "\n";
   rootFrame << "iv = " << iv << "\n";

   rootFrame << "ia*r - iv = " << res.mid() << "\n";
   rootFrame << "iv = " << iv.mid() << "\n";

   waitBt();
}


int main(int , char* [])
{
   openGW( 760,580,WHITE,BLACK);
   init_fpunit();

   if(DIM!=3) errorExit(" Wrong dimension!  \n Dimension in file vectalg/dimension.h should be 3.\n Change and recompile!");
   QRdecomp_test();
   diagonalize_test();
   norm_test();
   matrix_test();
   ztest_gauss();
   test_gauss();

   closeGW();
   round_nearest();
   return 0;
}

/// @}
